{
        This Code Was Created By Jan Koci 2002
        Visit My Site At http://koci.opengl.cz
}

// Unita, ktera nastavi zakladni hodnoty OpenGL a vystup vykresluje na fomular

unit KRenderForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;


type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);    //co se deje pri otvirani okna
    procedure FormDestroy(Sender: TObject);    //co se deje pri zavirani okna
    procedure FormKeyDown(Sender: TObject; var Key: Word;    //co se deje pri stisku nejakeho tlacitka
      Shift: TShiftState);
    procedure FormResize(Sender: TObject);    //co se deje pri zmene okna
  private
    { Private declarations }
     RC : HGLRC;    //Rendering Context - pripojuje OpenGL do HDC (Windows Device Context)
     m_iPixelFormat: integer;    //format vystupniho pixelu

  public
    { Public declarations }
    procedure Loop;    //procedura ktera prekresluje renderovaci okno v nekonecne smycce
    procedure RenderGLScene;    //procedura ktera vola funkce pro vykreslovani sceny
    procedure SetWindowStats(Width,Height,Color: integer);    //zobrazeni okna (normalni, nebo fullscreen)
    procedure SetPixelFormatDescriptor(DC: HDC);    //nastaveni pixeloveho formatu
    // DC = (Windows Device Context) - Kontext zarizeni, ktery se pripojuje do grafickeho rozhrani woken GDI (Graphics Device Interface)
    procedure InitWindow;    //nastaveni okna pro rendering
    procedure InitArea;    //nastaveni renderovaci sceny
  end;

var
  Form1: TForm1;



implementation

uses OpenGL, KThread, KDrawObjects, KTextures;    //pouzivej KThread (vlakno), KDrawObjects (kresleni objektu), KTextures(texturovani)



{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  //nastav OpenGL
  InitOpenGL;  //Inicializace OpenGL ovladacu (OpenGL32.dll, GLU32.dll)
  SetWindowStats(640, 480, 16);    //nastav rozliseni 640*480 16 bitovou hloubku barev
  InitWindow;    //nastav okno pro rendering
  InitArea;    //nastav zakladni renderovaci scenu


  //nastav vlakno - oddeleny proces, ktery bude nepretrzite volat proceduru Loop
  RenderThread := TMyThread.Create(True);    //vytvor prvni vlakno, ve kterem pobezi rendering sceny
  RenderThread.Priority := tpTimeCritical;    //vlakno ziska nejvyssi prioritu
                         //tpIdle;    //vlkno je provdno, kdy systm je neinn
                         //tpLowest;    //priorita vlakna je dva body pod normalem
                         //tpLower;    //priorita vlakna je jeden bod pod normalem
                         //tpNormal;    //vlakno ma normalni prioritu
                         //tpHigher;    //priorita vlakna je jeden bod nad normalem
                         //tpHighest;    //priorita vlakna je dva body nad normalem
  RenderThread.Run;    //rozjed toto vlakno
end;

procedure TForm1.SetWindowStats(Width,Height,Color: longint);
var
  dmScreenSettings: DEVMODE;    // Device Mode - mod zarizeni
begin
    // zeptej se, jestli budu chtit okno prepnout do fullscreenu
    if Application.MessageBox(
        'Would You Like To Run In Fullscreen Mode?',
        'Start FullScreen?',
        MB_YESNO + MB_ICONQUESTION) =IDYES
    then begin
      //pokud zadam Ano nastav tyto hodnoty
      ZeroMemory( @dmScreenSettings, sizeof( DEVMODE ) );    //vycisti pamet
      dmScreenSettings.dmSize := sizeof( DEVMODE );    //velikost struktury
      dmScreenSettings.dmPelsWidth  := Width;   //nastav sirku obrazovky
      dmScreenSettings.dmPelsHeight := Height;    //nastav vysku obrazovky
      dmScreenSettings.dmFields     := DM_PELSWIDTH or DM_PELSHEIGHT;
      dmScreenSettings.dmBitsPerPel := Color;    //nastav hloubku barev
      ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN);    //prepni do fullscreenu
      ShowCursor(false);    //schovej kurzor mysi
    end else begin
      //pokud nechci fullscreen
      Form1.WindowState := wsNormal;    //prepni maximalizovane okno na Normal
      Form1.BorderStyle := bsSizeable;    //a nastav, standardni okno
      Form1.ClientWidth := Width;    //precti sirku okna
      Form1.ClientHeight := Height;    //precti vysku okna
      Form1.Left := 0;    //nastav levy okraj okna na 0
      Form1.Top := 0;    //nastav horni okraj okna na 0
    end;
end;

procedure TForm1.InitArea;
begin
  glMatrixMode(GL_PROJECTION);    //nastav matici zobrazeni prostoru
  glLoadIdentity;    //pouzij tuto matici
  //Nastav uhel pohledu kamery, pomer vysky a sirky okna, nejblizsi a nejvzdalenejsi viditelny bod
  gluPerspective(45,Form1.ClientWidth / Form1.ClientHeight,0.1,100);
  glViewport(0,0,Form1.ClientWidth,Form1.ClientHeight);    //nastav velikost zobrazovaci plochy
end;


procedure TForm1.SetPixelFormatDescriptor;    //nastaveni pixeloveho formatu
var
  pfd: TPIXELFORMATDESCRIPTOR;
begin
  with pfd do begin
    nSize := sizeof(TPIXELFORMATDESCRIPTOR);    //velikost descriptoru
    nVersion := 1;    //cislo verze
    dwFlags := PFD_DRAW_TO_WINDOW or
                  PFD_SUPPORT_OPENGL or
                  PFD_DOUBLEBUFFER;    //formaty podporujici wokna, OpenGL, nastaveni doublebufferu
    iPixelType := PFD_TYPE_RGBA;    //pouzivej barvy ve formatu RGBA
    cColorBits := 24;    //buffer barev
    cDepthBits   := 32;    //32Bit Z-Buffer (Depth buffer)
    iLayerType   := PFD_MAIN_PLANE;    //vykreslovaci vrstva
  end;

  m_iPixelFormat := ChoosePixelFormat(DC, @pfd);    //vyber nejvhodnejsi pixelovy format
  SetPixelFormat(DC, m_iPixelFormat,@pfd);    //a tento pixelovy format nastav
end;

procedure TForm1.Loop;
begin
  //pripoj RC (OpenGL Rendering Context) do DC (Windows Device Context), ktery komunikuje s wokenim GDI (Graphics Device Interface)
  wglMakeCurrent(Form1.Canvas.Handle,RC);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);    //vycisti obrazovkovy buffer
  RenderGLScene;   //volej proceduru pro rendering objektu
  SwapBuffers(Form1.Canvas.Handle);    //prohod buffery (double burrefing)
  wglMakeCurrent(0,0);    //vynuluj RC (OpenGL Rendering Context)
end;

procedure TForm1.InitWindow;
begin
  SetPixelFormatDescriptor(Form1.Canvas.Handle);    //nastav format vystupniho pixelu
  RC := wglCreateContext(Form1.Canvas.Handle);     //vytvor RC (OpenGL Rendering Context)
  //pripoj RC (OpenGL Rendering Context) do DC (Windows Device Context), ktery komunikuje s wokenim GDI (Graphics Device Interface)
  wglMakeCurrent(Form1.Canvas.Handle,RC);
  glClearColor(0.0,0.0,0.0,0.0);    //nastav cerne pozadi sceny
    ////sem budu zadavat funkce, ktere se maji provadet jen pri startu aplikace////
      LoadGLTextures;    //vytvor textury
    ////konec zadavani funkci, ktere se maji provadet jen pri startu aplikace////
  wglMakeCurrent(0,0);    //vynuluj RC (OpenGL Rendering Context)
end;

procedure TForm1.RenderGLScene;
begin
   DrawTriangles;    //vykresli trojuhelniky
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if(Form1.BorderStyle = bsNone) then   //pokud je fullscreen
    ChangeDisplaySettings(DEVMODE(nil^), 0);    //nastav puvodni hodnotu rozliseni obrazovky

  ReleaseDC(Handle, Form1.Canvas.Handle);    //uvolni handle okna DC (Windows Device Context)
  wglMakeCurrent(0,0);    //vynuluj RC (OpenGL Rendering Context)
  wglDeleteContext(RC);    //vymaz RC (OpenGL Rendering Context)
  RenderThread.Done;  //ukonci vlakno
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if(Key=VK_ESCAPE) then    //jestlize klepnu na klavesu Esc
    Application.Terminate;    //ukonci aplikaci
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  //pripoj RC (OpenGL Rendering Context) do DC (Windows Device Context), ktery komunikuje s wokenim GDI (Graphics Device Interface)
  wglMakeCurrent(Form1.Canvas.Handle,RC);
  InitArea;    //nastav renderovaci scenu
  wglMakeCurrent(0,0);    //vynuluj RC (OpenGL Rendering Context)
end;


end.
